"""
Written by Selene Patani.

Copyright (c) 2023 European Union
Licenced under the MIT licence

Script made to export data related to harvested area/production of countries associated to the regulation and commodity tree products, for the web platform

"""


def main():
    from scripts.front_end.functions import (
        main_product_list,
        consistency_check_china_data,
        reporter_iso_codes,
        replace_zero_with_nan_values,
        save_file,
    )
    from biotrade.faostat import faostat
    import pandas as pd

    # Obtain the main product codes
    main_product_list = main_product_list(["crop_production", "forestry_production"])
    # Select quantities from Faostat db for crop data for all countries (code < 1000)
    crop_data = faostat.db.select(
        table="crop_production",
        product_code=main_product_list,
        element=["production", "area_harvested", "stocks"],
    )
    # Select wood production data
    wood_data = faostat.db.select(
        table="forestry_production",
        product_code=main_product_list,
        element=["production"],
    )
    # Merge data
    crop_data = pd.concat([crop_data, wood_data], ignore_index=True)
    crop_data = crop_data[crop_data["reporter_code"] < 1000]
    # China Mainland + Taiwan data
    df_china = consistency_check_china_data(crop_data)
    # Add China data to crop_data (exclude Taiwan data)
    crop_data = pd.concat(
        [crop_data[~(crop_data["reporter_code"] == 214)], df_china],
        ignore_index=True,
    )
    # Substitute faostat codes with iso3 codes
    crop_data = reporter_iso_codes(crop_data)
    # Columns to be retained
    column_list = ["reporter_code", "product_code", "period", "value", "unit"]
    # Harvested area data
    dropna_col = ["value"]
    crop_data = replace_zero_with_nan_values(crop_data, dropna_col)
    crop_data = crop_data.dropna(subset=dropna_col)
    harvested_area = crop_data[crop_data["element"] == "area_harvested"][column_list]
    # Production data
    production = crop_data[crop_data["element"].isin(["production", "stocks"])][column_list]
    # Save csv files to env variable path or into biotrade data folder
    save_file(harvested_area, "harvested_area_annual_variation.csv")
    save_file(production, "production_annual_variation.csv")


# Needed to avoid running module when imported
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
Written by Paul Rougieux and Selene Patani.

Copyright (c) 2023 European Union
Licenced under the MIT licence

JRC biomass Project.
Unit D1 Bioeconomy.

Specify the module dir to make it possible to access config data. Specify the
location of biotrade_data folder and the database connection URL for all
components of the system.

Usage:

    >>> from biotrade import module_dir
    >>> from biotrade import data_dir
    >>> from biotrade import database_url

"""

from pathlib import Path
import os

__version__ = "0.1.1"

module_dir = Path(__file__).resolve().parent

# Where is the data, default case
data_dir = Path.home() / "repos/forobs/biotrade_data/"

# But you can override that with an environment variable
if os.environ.get("BIOTRADE_DATA"):
    data_dir = Path(os.environ["BIOTRADE_DATA"])

# Create if not existing
if not data_dir.exists():
    if os.environ.get("BIOTRADE_SKIP_CONFIRMATION"):
        data_dir.mkdir(parents=True)
    else:
        msg = f"Create {data_dir}?"
        if input(msg + "Please confirm [y/n]:") == "y":
            data_dir.mkdir(parents=True)
        else:
            print("Directory creation cancelled.")

# Database connection URL, default case
database_url = None
# But you can override it with an environment variable
if os.environ.get("BIOTRADE_DATABASE_URL"):
    database_url = os.environ["BIOTRADE_DATABASE_URL"]



"""
Written by Selene Patani.

Copyright (c) 2023 European Union
Licenced under the MIT licence

Script made to export trends related to harvested area/production of countries associated to the regulation and commodity tree products, for the web platform

"""


def main():
    import sys
    from biotrade.faostat import faostat
    import pandas as pd
    import numpy as np
    from scripts.front_end.functions import (
        consistency_check_china_data,
        main_product_list,
        reporter_iso_codes,
        replace_zero_with_nan_values,
        trend_analysis,
        save_file,
    )

    # Default is using multi process
    multi_process = True
    # If module imported, avoid spawning in Windows
    if __name__ != "__main__":
        if sys.platform.startswith("win"):
            multi_process = False
    # Obtain the main product codes
    main_product_list = main_product_list(
        ["crop_production", "forestry_production"]
    )
    # Select quantities from Faostat db for crop data for all countries (code < 1000)
    crop_data = faostat.db.select(
        table="crop_production",
        product_code=main_product_list,
        element=["production", "area_harvested", "stocks"],
    )
    # Select wood production data
    wood_data = faostat.db.select(
        table="forestry_production",
        product_code=main_product_list,
        element=["production"],
    )
    # Merge data
    crop_data = pd.concat([crop_data, wood_data], ignore_index=True)
    crop_data = crop_data[crop_data["reporter_code"] < 1000]
    # China Mainland + Taiwan data
    df_china = consistency_check_china_data(crop_data)
    # Add China data to crop_data (exclude Taiwan data)
    crop_data = pd.concat(
        [crop_data[~(crop_data["reporter_code"] == 214)], df_china],
        ignore_index=True,
    )
    # Substitute faostat codes with iso3 codes
    crop_data = reporter_iso_codes(crop_data)
    # Consider data after 1985 to calculate trends of last year
    crop_data = crop_data[crop_data["year"] > 1985]
    # Perform trend analysis
    df = trend_analysis(crop_data, multi_process=multi_process)
    # Columns to be retained
    column_list = [
        "reporter_code",
        "product_code",
        "period",
        "period_change",
        "period_regression",
        "relative_change",
        "absolute_change",
        "mk_slope",
        "mk_significance_flag",
        "unit",
    ]
    # Drop nan values
    dropna_col = ["relative_change", "absolute_change", "mk_slope"]
    df = replace_zero_with_nan_values(df, dropna_col)
    # Put nan to period and significance columns when relative change or slope is 0
    # To avoid inconsistencies during the replace zero with nan
    selector = df.mk_slope.isnull()
    df.loc[selector, "period_regression"] = np.nan
    df.loc[selector, "mk_significance_flag"] = np.nan
    selector = df.relative_change.isnull() & df.absolute_change.isnull()
    df.loc[selector, "period_change"] = np.nan
    df = df.dropna(subset=dropna_col, how="all")
    # Harvested area data (only the most recent year of db)
    most_recent_year = sorted(df.year.unique(), reverse=True)[0]
    harvested_area = df[
        (df["element"] == "area_harvested") & (df["year"] == most_recent_year)
    ][column_list]
    # Production data (only the most recent year of db)
    production = df[
        (df["element"].isin(["production", "stocks"]))
        & (df["year"] == most_recent_year)
    ][column_list]
    save_file(harvested_area, "harvested_area_trends.csv")
    save_file(production, "production_trends.csv")


# Needed to avoid running module when imported
if __name__ == "__main__":
    main()



"""
Script which updates biotrade database
"""

from biotrade.faostat import faostat

faostat.pump.update(["country"])
# faostat.pump.update(["crop_production", "crop_trade"])
# faostat.pump.update(["forestry_production", "forestry_trade", "forest_land"])
# faostat.pump.update(["food_balance"])
# faostat.pump.update(["land_use", "land_cover"])
